import csv
from logging import info
from typing import List

from airflow.providers.oracle.hooks.oracle import OracleHook as AirflowOracleHook


class OracleHook(AirflowOracleHook):
    def __init__(self, *args, **kwargs):
        super(OracleHook, self).__init__(*args, **kwargs)

    def export(
        self,
        sql: str,
        output_file_path: str,
        parameters: List[str] = None,
        include_headers: bool = True,
        batch_size: int = 10000,
        **kwargs,
    ) -> str:
        with self.get_conn() as oracle_conn:
            info("Oracle connection acquired")

            cursor = oracle_conn.cursor()
            info("Oracle cursor acquired")

            with open(output_file_path, "w") as csv_file:
                info(f"Output file opened")
                csv_writer = csv.writer(csv_file)

                cursor.execute(sql, parameters, **kwargs)

                if include_headers:
                    csv_writer.writerow([desc[0] for desc in cursor.description])

                processed_row = 0
                rows = cursor.fetchmany(batch_size)
                
                while len(rows) > 0:
                    csv_writer.writerows(rows)
                    processed_row += len(rows)

                    rows = cursor.fetchmany(batch_size)

        info(f"Oracle export finished, total rows extracted: {processed_row}")



from logging import warn
import os

from airflow.secrets import BaseSecretsBackend


class FileSystemSecretsBackend(BaseSecretsBackend):
    def __init__(
        self,
        connections_path="/opt/airflow/connections",
        variables_path="/opt/airflow/variables",
        **kwargs,
    ):
        super().__init__(**kwargs)
        self.connections_path = connections_path
        self.variables_path = variables_path

    def get_conn_uri(self, conn_id: str):
        filepath = os.path.join(self.connections_path, conn_id)
        if not os.path.exists(filepath):
            warn(
                f"{filepath} does not exist. Please check your secrets backend connections_path of the connection id."
            )
            return None
        with open(filepath) as file:
            return file.read()

    def get_variable(self, key: str):
        filepath = os.path.join(self.variables_path, key)
        if not os.path.exists(filepath):
            warn(
                f"{filepath} does not exist. Please check your secrets backend variables_path of the variable key."
            )
            return None
        with open(filepath) as file:
            return file.read()



from logging import info
from typing import Dict

from airflow.models import BaseOperator
from airflow.models.param import ParamsDict
from airflow.plugins_manager import AirflowPlugin

from  reign_airflow_utils.hooks.oracle_hook import OracleHook


class OracleOperatorDumpExpert(BaseOperator):
    template_fields = ("output_file_path", "parameters")
    template_ext = (".sql",)

    def __init__(
        self, 
        oracle_conn_id: str, 
        sql_path: str, 
        output_file_path: str, 
        parameters: Dict = None, 
        batch_size: int = 10000,
        *args, 
        **kwargs
    ):
        super(OracleOperatorDumpExpert, self).__init__(*args, **kwargs)

        self.oracle_conn_id = oracle_conn_id
        self.sql_path = sql_path
        self.output_file_path = output_file_path
        self.parameters = parameters
        self.params = ParamsDict(parameters)
        self.batch_size = batch_size

    def execute(self, context):
        with open(self.sql_path, "r") as file:
            sql = file.read()

        if not sql:
            raise IOError("Missed query")

        info(f'Dumping Oracle query results to local file {self.output_file_path} using query "{sql}"')

        hook = OracleHook(oracle_conn_id=self.oracle_conn_id)
        hook.export(sql=sql, output_file_path=self.output_file_path, parameters=self.params, batch_size=self.batch_size)


class OracleOperatorDumpExpertPlugin(AirflowPlugin):
    name = "oracle_operator_dump_expert"
    operators = [OracleOperatorDumpExpert]
